/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FullWizard.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.fullwizard;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.border.EmptyBorder;

import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.preferences.PreferencesFrameDescriptor;
import org.ow2.jonas.autostart.starterfull.FullStarter;
import org.ow2.jonas.autostart.stop.StopFrameDescriptor;

/**
 * This class implements the wizard dialog.
 * @author skabore
 */
public class FullWizard extends JFrame implements PropertyChangeListener {

    /**
     * Indicates that the 'Finish' button was pressed to close the dialog.
     */
    public static final int FINISH_RETURN_CODE = 0;

    /**
     * Indicates that the 'Cancel' button was pressed to close the dialog, or
     * the user pressed the close box in the corner of the window.
     */
    public static final int CANCEL_RETURN_CODE = 1;

    /**
     * Indicates that the dialog closed due to an internal error.
     */
    public static final int ERROR_RETURN_CODE = 2;

    /**
     * Indicates that the 'Set config' button was pressed to close the dialog,
     * or the user pressed the close box in the corner of the window.
     */
    public static final int CONFIG_RETURN_CODE = 3;

    /**
     * Indicates that the 'Preferences' button was pressed to close the dialog,
     * or the user pressed the close box in the corner of the window.
     */
    public static final int PREFERENCES_RETURN_CODE = 4;

    /**
     * Indicates that the 'Stop' button was pressed to close the dialog, or the
     * user pressed the close box in the corner of the window.
     */
    public static final int STOP_RETURN_CODE = 5;

    /**
     * The String-based action command for the 'Next' button.
     */
    public static final String NEXT_BUTTON_ACTION_COMMAND = "NextButtonActionCommand";

    /**
     * The String-based action command for the 'Back' button.
     */
    public static final String BACK_BUTTON_ACTION_COMMAND = "BackButtonActionCommand";

    /**
     * The String-based action command for the 'Cancel' button.
     */
    public static final String CANCEL_BUTTON_ACTION_COMMAND = "CancelButtonActionCommand";

    /**
     * The String-based action command for the 'Set Config' button.
     */
    public static final String CONFIG_BUTTON_ACTION_COMMAND = "SetConfigButtonActionCommand";

    /**
     * The String-based action command for the 'Preferences' button.
     */
    public static final String PREFERENCES_BUTTON_ACTION_COMMAND = "PreferencesButtonActionCommand";

    /**
     * The String-based action command for the 'Stop' button.
     */
    public static final String STOP_BUTTON_ACTION_COMMAND = "StopButtonActionCommand";

    /**
     * The back button text.
     */
    static String BACK_TEXT;

    /**
     * The next button text.
     */
    static String NEXT_TEXT;

    /**
     * The finish button text.
     */
    static String FINISH_TEXT;

    /**
     * The cancel button text.
     */
    static String CANCEL_TEXT;

    /**
     * The config button text.
     */
    static String CONFIG_TEXT;

    /**
     * The prefernces button text.
     */
    static String PREFERNCES_TEXT;

    /**
     * The stop button text.
     */
    static String STOP_TEXT;

    /**
     * The wizard model.
     */
    private FullWizardModel wizardModel;

    /**
     * The wizard controller.
     */
    private FullWizardController wizardController;

    /**
     * The wizard dialog.
     */
    // private JDialog wizardDialog;
    private JFrame wizardDialog;

    /**
     * The card panel.
     */
    private JPanel cardPanel;

    /**
     * The card layout.
     */
    private CardLayout cardLayout;

    /**
     * The back button.
     */
    private JButton backButton;

    /**
     * The next button.
     */
    private JButton nextButton;

    /**
     * The cancel button.
     */
    private JButton cancelButton;

    /**
     * The config button.
     */
    private JButton configButton;

    /**
     * The stop button.
     */
    private JButton stopButton;

    /**
     * The preferences button.
     */
    private JButton preferencesButton;

    /**
     * The button box.
     */
    private Box buttonBox;

    /**
     * Fibe constant.
     */
    private final static int FIVE_CONSTANT = 5;

    /**
     * The button box.
     */
    private int horizontalStrut = FIVE_CONSTANT;

    /**
     * The return code value.
     */
    private int returnCode;

    /**
     * The current frame value. 0 = GettingJonasFieldFromWebsiteOptionFrame 1 =
     * GettingJonasFieldFromDiskOptionFrame 2 =
     * GettingJonasFieldFromZipFileOptionFrame 10 =
     * RunningJonasFieldFromEmbeddedFrame 11 = RunningJonasFieldFromDiskFrame
     */
    private int currentFrame;

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The starter librairy.
     */
    private FullStarter starter;

    /**
     * The configuration.
     */
    private Configuration configuration;

    /**
     * The preferencesFrameDescriptor.
     */
    private PreferencesFrameDescriptor preferencesFrameDescriptor;

    /**
     * The stopFrameDescriptor.
     */
    private StopFrameDescriptor stopFrameDescriptor;

    /**
     * The temporary working directory.
     */
    private File workDirectory;

    /**
     * Default constructor. This method creates a new FullWizardModel object and
     * passes it into the overloaded constructor.
     */
    public FullWizard() {
        this((Frame) null);
        preferencesFrameDescriptor = null;
        stopFrameDescriptor = null;
        workDirectory = null;
    }

    /**
     * This method accepts a java.awt.Dialog object as the javax.swing.JDialog's
     * parent.
     * @param owner The java.awt.Dialog object that is the owner of this dialog.
     */
    public FullWizard(final Dialog owner) {
        wizardModel = new FullWizardModel();
        wizardDialog = new JFrame();
        initComponents();
        preferencesFrameDescriptor = null;
        stopFrameDescriptor = null;
        workDirectory = null;
    }

    /**
     * This method accepts a java.awt.Frame object as the javax.swing.JDialog's
     * parent.
     * @param owner The java.awt.Frame object that is the owner of the
     *        javax.swing.JDialog.
     */
    public FullWizard(final Frame owner) {
        wizardModel = new FullWizardModel();
        wizardDialog = new JFrame();
        initComponents();
        preferencesFrameDescriptor = null;
        stopFrameDescriptor = null;
        workDirectory = null;
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return builder;
    }

    /**
     * Set the builder.
     * @param builder the new builder
     */
    public void setBuilder(final Builder builder) {
        this.builder = builder;
    }

    /**
     * @return starter.
     */
    public File getWorkDirectory() {
        workDirectory = builder.getWorkdirectory();
        if (workDirectory == null) {
            if (starter != null) {
                workDirectory = starter.getWorkdirectory();
            } else {
                builder.createTmpWorkDirectory();
                workDirectory = builder.getWorkdirectory();
            }
        }
        return workDirectory;
    }

    /**
     * @return starter.
     */
    public FullStarter getStarter() {
        return starter;
    }

    /**
     * Set the starter.
     * @param starter the new starter
     */
    public void setStarter(final FullStarter starter) {
        this.starter = starter;
    }

    /**
     * Returns an instance of the JDialog that this class created. This is
     * useful in the event that you want to change any of the JDialog parameters
     * manually.
     * @return The JDialog instance that this class created.
     */
    public JFrame getDialog() {
        return wizardDialog;
    }

    /**
     * Sets the title of the generated javax.swing.JDialog.
     * @param s The title of the dialog.
     */
    @Override
    public void setTitle(final String s) {
        wizardDialog.setTitle(s);
    }

    /**
     * Returns the current title of the generated dialog.
     * @return The String-based title of the generated dialog.
     */
    @Override
    public String getTitle() {
        return wizardDialog.getTitle();
    }

    /**
     * @return horizontalStrut value.
     */
    public int getHorizontalStrut() {
        return horizontalStrut;
    }

    /**
     * Set the horizontalStrut value.
     * @param horizontalStrut new value
     */
    public void setHorizontalStrut(final int horizontalStrut) {
        this.horizontalStrut = horizontalStrut;
    }

    /**
     * Convienence method that displays a modal wizard dialog and blocks until
     * the dialog has completed.
     * @return Indicates how the dialog was closed. Compare this value against
     *         the RETURN_CODE constants at the beginning of the class.
     */
    public int showModalDialog() {
        wizardDialog.setLocation(
                (Toolkit.getDefaultToolkit().getScreenSize().width - wizardDialog.getPreferredSize().width) / 2, (Toolkit
                        .getDefaultToolkit().getScreenSize().height - wizardDialog.getPreferredSize().height) / 2);
        wizardDialog.pack();
        wizardDialog.setVisible(true);
        wizardDialog.setResizable(false);
        wizardDialog.setDefaultCloseOperation(EXIT_ON_CLOSE);
        return returnCode;
    }

    /**
     * Returns the current model of the wizard dialog.
     * @return A FullWizardModel instance, which serves as the model for the
     *         wizard dialog.
     */
    public FullWizardModel getModel() {
        return wizardModel;
    }

    /**
     * Add a Component as a panel for the wizard dialog by registering its
     * FullWizardPanelDescriptor object. Each panel is identified by a unique
     * Object-based identifier (often a String), which can be used by the
     * setCurrentPanel() method to display the panel at runtime.
     * @param id An Object-based identifier used to identify the
     *        FullWizardPanelDescriptor object.
     * @param panel The FullWizardPanelDescriptor object which contains helpful
     *        information about the panel.
     */
    public void registerWizardPanel(final Object id, final FullWizardPanelDescriptor panel) {

        // Add the incoming panel to our JPanel display that is managed by
        // the CardLayout layout manager.

        cardPanel.add(panel.getPanelComponent(), id);
        // cardPanel.add(panel.getPanelComponent(), BorderLayout.CENTER);
        // Set a callback to the current wizard.

        panel.setWizard(this);

        // Place a reference to it in the model.

        wizardModel.registerPanel(id, panel);

    }

    /**
     * Displays the panel identified by the object passed in. This is the same
     * Object-based identified used when registering the panel.
     * @param id The Object-based identifier of the panel to be displayed.
     */
    public void setCurrentPanel(final Object id) {

        // Get the hashtable reference to the panel that should
        // be displayed. If the identifier passed in is null, then close
        // the dialog.

        if (id == null) {
            close(ERROR_RETURN_CODE);
        }

        FullWizardPanelDescriptor oldPanelDescriptor = wizardModel.getCurrentPanelDescriptor();
        if (oldPanelDescriptor != null) {
            oldPanelDescriptor.aboutToHidePanel();
        }

        wizardModel.setCurrentPanel(id);
        wizardModel.getCurrentPanelDescriptor().aboutToDisplayPanel();
        // Show the panel in the dialog.
        cardLayout.show(cardPanel, id.toString());
        wizardModel.getCurrentPanelDescriptor().displayingPanel();
    }

    /**
     * Method used to listen for property change events from the model and
     * update the dialog's graphical components as necessary.
     * @param evt PropertyChangeEvent passed from the model to signal that one
     *        of its properties has changed value.
     */
    public void propertyChange(final PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(FullWizardModel.CURRENT_PANEL_DESCRIPTOR_PROPERTY)) {
            wizardController.resetButtonsToPanelRules();
        } else if (evt.getPropertyName().equals(FullWizardModel.NEXT_FINISH_BUTTON_TEXT_PROPERTY)) {
            nextButton.setText(evt.getNewValue().toString());
        } else if (evt.getPropertyName().equals(FullWizardModel.BACK_BUTTON_TEXT_PROPERTY)) {
            backButton.setText(evt.getNewValue().toString());
        } else if (evt.getPropertyName().equals(FullWizardModel.CANCEL_BUTTON_TEXT_PROPERTY)) {
            cancelButton.setText(evt.getNewValue().toString());
        } else if (evt.getPropertyName().equals(FullWizardModel.NEXT_FINISH_BUTTON_ENABLED_PROPERTY)) {
            nextButton.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
        } else if (evt.getPropertyName().equals(FullWizardModel.BACK_BUTTON_ENABLED_PROPERTY)) {
            backButton.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
        } else if (evt.getPropertyName().equals(FullWizardModel.CANCEL_BUTTON_ENABLED_PROPERTY)) {
            cancelButton.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
        }
    }

    /**
     * Retrieves the last return code set by the dialog.
     * @return An integer that identifies how the dialog was closed. See the
     *         *_RETURN_CODE constants of this class for possible values.
     */
    public int getReturnCode() {
        return returnCode;
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @return A boolean indicating if the button is enabled.
     */
    public boolean getBackButtonEnabled() {
        return wizardModel.getBackButtonEnabled().booleanValue();
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @param newValue The new enabled status of the button.
     */
    public void setBackButtonEnabled(final boolean newValue) {
        wizardModel.setBackButtonEnabled(new Boolean(newValue));
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @return A boolean indicating if the button is enabled.
     */
    public boolean getNextFinishButtonEnabled() {
        return wizardModel.getNextFinishButtonEnabled().booleanValue();
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @param newValue The new enabled status of the button.
     */
    public void setNextFinishButtonEnabled(final boolean newValue) {
        wizardModel.setNextFinishButtonEnabled(new Boolean(newValue));
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @return A boolean indicating if the button is enabled.
     */
    public boolean getCancelButtonEnabled() {
        return wizardModel.getCancelButtonEnabled().booleanValue();
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @param newValue The new enabled status of the button.
     */
    public void setCancelButtonEnabled(boolean newValue) {
        wizardModel.setCancelButtonEnabled(new Boolean(newValue));
    }

    /**
     * Mirrors the FullWizardModel method of the same name.
     * @param newValue The new enabled status of the button.
     */
    public void setPreferencesButtonEnabled(final boolean newValue) {
        wizardModel.setPreferencesButtonEnabled(new Boolean(newValue));
    }

    /**
     * @return nextButton.
     */
    public JButton getNextButton() {
        return nextButton;
    }

    /**
     * @return configButton.
     */
    public JButton getConfigButton() {
        return configButton;
    }

    /**
     * @return preferencesButton.
     */
    public JButton getPreferencesButton() {
        return preferencesButton;
    }

    /**
     * @return stopButton.
     */
    public JButton getStopButton() {
        return stopButton;
    }

    /**
     * @return currentFrame.
     */
    public int getCurrentFrame() {
        return currentFrame;
    }

    /**
     * @return configuration.
     */
    public Configuration getConfiguration() {
        return configuration;
    }

    /**
     * Set the configuration
     * @param configuration the configuration new value.
     */
    public void setConfiguration(final Configuration configuration) {
        this.configuration = configuration;
    }

    /**
     * Set the currrentFrame
     * @param currentFrame the currentFrame new value
     */
    public void setCurrentFrame(final int currentFrame) {
        this.currentFrame = currentFrame;
    }

    /**
     * @return preferencesFrameDescriptor
     */
    public PreferencesFrameDescriptor getPreferencesFrameDescriptor() {
        return preferencesFrameDescriptor;
    }

    /**
     * Set the preferencesFrameDescriptor.
     * @param preferencesFrameDescriptor the preferencesFrameDescriptor new
     *        value
     */
    public void setPreferencesFrameDescriptor(final PreferencesFrameDescriptor preferencesFrameDescriptor) {
        this.preferencesFrameDescriptor = preferencesFrameDescriptor;
    }

    /**
     * @return stopFrameDescriptor
     */
    public StopFrameDescriptor getStopFrameDescriptor() {
        return stopFrameDescriptor;
    }

    /**
     * Set the stopFrameDescriptor.
     * @param stopFrameDescriptor the stopFrameDescriptor new value
     */
    public void setStopFrameDescriptor(final StopFrameDescriptor stopFrameDescriptor) {
        this.stopFrameDescriptor = stopFrameDescriptor;
    }

    /**
     * Closes the dialog and sets the return code to the integer parameter.
     * @param code The return code.
     */
    public void close(final int code) {
        returnCode = 0;
    }

    /**
     * This method initializes the components for the wizard dialog: it creates
     * a JDialog as a CardLayout panel surrounded by a small amount of space on
     * each side, as well as three buttons at the bottom.
     */
    private void initComponents() {

        wizardModel.addPropertyChangeListener(this);
        wizardController = new FullWizardController(this);

        wizardDialog.getContentPane().setLayout(new BorderLayout());

        JPanel buttonPanel = new JPanel();
        JSeparator separator = new JSeparator();
        buttonBox = new Box(BoxLayout.X_AXIS);

        cardPanel = new JPanel();
        cardPanel.setBorder(new EmptyBorder(new Insets(5, 10, 5, 10)));

        cardLayout = new CardLayout();
        cardPanel.setLayout(cardLayout);

        backButton = new JButton("Back");
        nextButton = new JButton("Next");
        cancelButton = new JButton("Cancel");
        configButton = new JButton("Set config");
        preferencesButton = new JButton("Preferences");
        stopButton = new JButton("Stop");
        backButton.setMnemonic(KeyEvent.VK_B);
        nextButton.setMnemonic(KeyEvent.VK_N);
        cancelButton.setMnemonic(KeyEvent.VK_C);
        configButton.setMnemonic(KeyEvent.VK_G);
        preferencesButton.setMnemonic(KeyEvent.VK_P);
        stopButton.setMnemonic(KeyEvent.VK_O);

        backButton.setActionCommand(BACK_BUTTON_ACTION_COMMAND);
        nextButton.setActionCommand(NEXT_BUTTON_ACTION_COMMAND);
        cancelButton.setActionCommand(CANCEL_BUTTON_ACTION_COMMAND);
        configButton.setActionCommand(CONFIG_BUTTON_ACTION_COMMAND);
        preferencesButton.setActionCommand(PREFERENCES_BUTTON_ACTION_COMMAND);
        stopButton.setActionCommand(STOP_BUTTON_ACTION_COMMAND);

        backButton.addActionListener(wizardController);
        nextButton.addActionListener(wizardController);
        cancelButton.addActionListener(wizardController);
        configButton.addActionListener(wizardController);
        preferencesButton.addActionListener(wizardController);
        stopButton.addActionListener(wizardController);

        buttonPanel.setLayout(new BorderLayout());
        buttonPanel.add(separator, BorderLayout.NORTH);

        buttonBox.setBorder(new EmptyBorder(new Insets(5, 10, 5, 10)));

        buttonBox.add(backButton);
        buttonBox.add(Box.createHorizontalStrut(horizontalStrut));

        buttonBox.add(nextButton);
        buttonBox.add(Box.createHorizontalStrut(horizontalStrut));

        buttonBox.add(stopButton);
        stopButton.setVisible(false);
        // buttonBox.add(Box.createHorizontalStrut(10));

        buttonBox.add(configButton);
        configButton.setVisible(false);
        buttonBox.add(Box.createHorizontalStrut(horizontalStrut));
        buttonBox.add(preferencesButton);
        buttonBox.add(Box.createHorizontalStrut(horizontalStrut));
        preferencesButton.setVisible(false);
        buttonBox.add(cancelButton);

        buttonPanel.add(buttonBox, BorderLayout.EAST);
        cardPanel.setBorder(new EmptyBorder(new Insets(5, 10, 5, 10)));
        wizardDialog.getContentPane().add(cardPanel, BorderLayout.CENTER);
        wizardDialog.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    }

    /**
     * If the user presses the close box on the dialog's window, treat it as a
     * cancel.
     * @param event The event passed in from AWT.
     */
    public void windowClosing(final WindowEvent event) {
        returnCode = EXIT_ON_CLOSE;
    }

    /**
     * Show a message dialog.
     * @param message the message to print.
     */
    public void showMessage(final String message) {
        JOptionPane.showMessageDialog(this, message, "Informations", JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Initialize the buttons text.
     */
    static {
        BACK_TEXT = "Back";
        NEXT_TEXT = "Next";
        CANCEL_TEXT = "Cancel";
        FINISH_TEXT = "Finish";
        CONFIG_TEXT = "Set config";
        STOP_TEXT = "Stop";
    }
}
